{$I newton_base.inc}
{%mainunit ../orca_scene3d.pas}

type
  TD3NewtonPhysics = class(TD3Physics)
  private
    FNeedSimulation: boolean;
    FDestroyBodyList: TList;
    FBodies: TList;
    FJoins: TList;
    FNewtonExists: boolean;
  protected
    function CheckWorld(const AOwner: TD3VisualObject): NewtonWorld;
    procedure IntDestroyBody(ABody: integer);
  public
    constructor Create(const Scene: TD3Scene); override;
    destructor Destroy; override;
    function CreateWorld: integer; override;
    procedure DestroyWorld(const AWorld: integer); override;
    procedure UpdateWorld(const AWorld: integer; const DeltaTime: single); override;
    function Collise(const AOwner: TD3VisualObject; var AList: TList): integer; override;
    function ComplexCollise(const AOwner: TD3VisualObject; var AList: TList): integer; override;
    function ObjectByBody(const Body: integer): TD3VisualObject; override;
    function CreateBox(const AOwner: TD3VisualObject; const ASize: TD3Vector): integer; override;
    function CreateSphere(const AOwner: TD3VisualObject; const ASize: TD3Vector): integer; override;
    function CreateCone(const AOwner: TD3VisualObject; const ASize: TD3Vector): integer; override;
    function CreateCylinder(const AOwner: TD3VisualObject; const ASize: TD3Vector): integer; override;
    procedure DestroyBody(var ABody: integer); override;
    procedure AddForce(const AOwner: TD3VisualObject; const Force: TD3Vector); override;
    procedure Explode(const AWorld: integer; const Position: TD3Vector; const Radius, Force: single); override;
    procedure SetBodyMatrix(const Body: integer; const M: TD3Matrix); override;
    function GetBodyMatrix(const Body: integer): TD3Matrix; override;
  end;

const
  ScreenDepth = 10.0;
  cEps = 0.0000000001;

var
  NewtonLoaded: integer = 0;
  ExplodeCenter: TD3Vector;
  ExplodeForce: single;
  ColliseColList: array [0..1000] of NewtonBody;
  ColliseColListCount: integer;
  ComplexColList: array of NewtonBody;

type
  TCollisionRec = record
    O1, O2: Pointer;
  end;

procedure PhysicsApplyForceAndTorque(const Body: NewtonBody); cdecl;
var
  Vec: TD3Vector;
  M: TD3Matrix;
  Obj: TD3VisualObject;
begin
  NewtonBodyGetMatrix(Body, @M);
  Obj := TD3VisualObject(NewtonBodyGetUserData(Body));
  if Obj <> nil then
  begin
    if Obj.Projection = d3ProjectionScreen then
    begin
      M.M43 := 0;
      Vec.X := 0;
      Vec.Y := 50;
      NewtonBodySetForce(Body, @Vec);
    end
    else
    begin
      Vec.X := 0;
      Vec.Y := 0;
      Vec.Z := -9;
      NewtonBodySetForce(Body, @Vec);
    end;
  end;
  NewtonBodySetMatrix(Body, @M);
end;

constructor TD3NewtonPhysics.Create;
begin
  NewtonLoaded := NewtonLoaded + 1;
  FNewtonExists := True;
  inherited;
  FBodies := TList.Create;
  FJoins := TList.Create;
end;

destructor TD3NewtonPhysics.Destroy;
begin
  if FDestroyBodyList <> nil then
    FDestroyBodyList.Free;
  FBodies.Free;
  FJoins.Free;
  inherited;
  NewtonLoaded := NewtonLoaded - 1;
end;

function TD3NewtonPhysics.CreateWorld: integer;
begin
  Result := 0;
  if not FNewtonExists then
    Exit;
  Result := integer(NewtonCreate);
  NewtonSetSolverModel(NewtonWorld(Result), 0);
  NewtonSetFrictionModel(NewtonWorld(Result), 0);
end;

procedure TD3NewtonPhysics.DestroyWorld(const AWorld: integer);
begin
  if not FNewtonExists then
    Exit;
  NewtonDestroy(NewtonWorld(AWorld));
end;

procedure ColliseBodyIterAABB(const Body: NewtonBody); cdecl;
begin
  ColliseColList[ColliseColListCount] := body;
  System.Inc(ColliseColListCount);
end;

function TD3NewtonPhysics.Collise(const AOwner: TD3VisualObject; var AList: TList): integer;
var
  I: integer;
  AttrA: array[0..1] of dLong;
  AttrB: array[0..1] of dLong;
  B1, B2: NewtonBody;
  C1, C2: NewtonCollision;
  M1, M2: TD3Matrix;
  Obj: TD3VisualObject;
  C, P, N: TD3Vector;
  A, B: TD3Vector;
begin
  Result := 0;
  if AOwner.Body = 0 then
    Exit;
  if not AOwner.Visible then
    Exit;
  if AList.IndexOf(AOwner) >= 0 then
    Exit;
  B1 := NewtonBody(AOwner.Body);
  if (FDestroyBodyList <> nil) and (FDestroyBodyList.IndexOf(B1) >= 0) then
    Exit;
  NewtonBodyGetAABB(B1, @A, @B);
  ColliseColListCount := 0;
  NewtonWorldForEachBodyInAABBDo(CheckWorld(AOwner), @A, @B, @ColliseBodyIterAABB, nil);
  for I := 0 to ColliseColListCount - 1 do
  begin
    B2 := ColliseColList[I];
    if B1 = B2 then
      Continue;
    if (FDestroyBodyList <> nil) and (FDestroyBodyList.Count > 0) then
    begin
      if FDestroyBodyList.IndexOf(B2) >= 0 then
      begin
        Continue;
      end;
    end;
    Obj := ObjectByBody(integer(B2));
    if not Obj.Visible then
      Continue;
    C1 := NewtonBodyGetCollision(B1);
    C2 := NewtonBodyGetCollision(B2);
    NewtonBodyGetMatrix(B1, @M1);
    NewtonBodyGetMatrix(B2, @M2);
    if NewtonCollisionCollide(CheckWorld(AOwner), 1, C1, @M1, C2, @M2, @C, @N, @P, @AttrA, @AttrB, 0) > 0 then
    begin
      N.W := 1;
      P.W := 1;
      AOwner.Collision(Obj, P, N);
    end;
  end;
end;

procedure ComplexBodyIterAABB(const Body: NewtonBody); cdecl;
begin
  SetLength(ComplexColList, Length(ComplexColList) + 1);
  ComplexColList[High(ComplexColList)] := Body;
end;

function TD3NewtonPhysics.ComplexCollise(const AOwner: TD3VisualObject; var AList: TList): integer;
var
  I: integer;
  AttrA: array[0..1] of dLong;
  AttrB: array[0..1] of dLong;
  B1, B2: NewtonBody;
  C1, C2: NewtonCollision;
  M1, M2: TD3Matrix;
  Obj: TD3VisualObject;
  C, P, N: TD3Vector;
  A, B: TD3Vector;
begin
  Result := 0;
  if AOwner.Body = 0 then
    Exit;
  if not AOwner.Visible then
    Exit;
  if AList.IndexOf(AOwner) >= 0 then
    Exit;
  SetLength(ComplexColList, 0);
  AList.Add(AOwner);
  B1 := NewtonBody(AOwner.Body);
  if (FDestroyBodyList <> nil) and (FDestroyBodyList.IndexOf(B1) >= 0) then
    Exit;
  NewtonBodyGetAABB(B1, @A, @B);
  NewtonWorldForEachBodyInAABBDo(CheckWorld(AOwner), @A, @B, @ComplexBodyIterAABB, nil);
  for I := 0 to High(ComplexColList) do
  begin
    B2 := ComplexColList[I];
    if B2 = B1 then
      Continue;
    if (FDestroyBodyList <> nil) and (FDestroyBodyList.Count > 0) then
    begin
      if FDestroyBodyList.IndexOf(B2) >= 0 then
      begin
        Continue;
      end;
    end;
    Obj := ObjectByBody(integer(ComplexColList[I]));
    if not Obj.Visible then
      Continue;
    C1 := NewtonBodyGetCollision(B1);
    C2 := NewtonBodyGetCollision(B2);
    NewtonBodyGetMatrix(B1, @M1);
    NewtonBodyGetMatrix(B2, @M2);
    if NewtonCollisionCollide(CheckWorld(AOwner), 1, C1, @M1, C2, @M2, @C, @N, @P, @AttrA, @AttrB, 0) > 0 then
    begin
      N.W := 1;
      P.W := 1;
      AOwner.Collision(Obj, P, N);
    end;
  end;
end;

procedure TD3NewtonPhysics.UpdateWorld(const AWorld: integer; const DeltaTime: single);
var
  I: integer;
  SIMT: single;
begin
  if not FNewtonExists then
    Exit;
  SIMT := FScene.Time;
  if FNeedSimulation then
  begin
    NewtonUpdate(NewtonWorld(AWorld), DeltaTime);
    if FDestroyBodyList <> nil then
    begin
      for I := FDestroyBodyList.Count - 1 downto 0 do
        IntDestroyBody(integer(FDestroyBodyList[I]));
      FDestroyBodyList.Clear;
    end;
  end;
end;

procedure BodyExplodeAABB(const Body: NewtonBody); cdecl;
var
  Ray, Force: TD3Vector;
  M: TD3Matrix;
begin
  NewtonBodyGetMatrix(Body, @M);
  if (Abs(M.M[3].X - ExplodeCenter.X) > cEps) or (Abs(M.M[3].Y - ExplodeCenter.Y) > cEps) or (Abs(M.M[3].Z - ExplodeCenter.Z) > cEps) then
  begin
    Ray := d3VectorScale(d3VectorNormalize(d3VectorSubtract(M.M[3], ExplodeCenter)), ExplodeForce);
    Force := d3Vector(10, 0, 0);
    NewtonBodyAddImpulse(Body, @Ray, @M.M[3]);
  end;
end;

procedure TD3NewtonPhysics.Explode(const AWorld: integer; const Position: TD3Vector; const Radius, Force: single);
var
  A, B: TD3Vector;
begin
  if not FNewtonExists then
    Exit;
  A.X := Position.X - Radius;
  A.Y := Position.Y - Radius;
  A.Z := Position.Z - Radius;
  B.X := Position.X + Radius;
  B.Y := Position.Y + Radius;
  B.Z := Position.Z + Radius;
  ExplodeCenter := Position;
  ExplodeForce := Force;
  NewtonWorldForEachBodyInAABBDo(NewtonWorld(AWorld), @A, @B, @BodyExplodeAABB, nil);
end;

procedure TD3NewtonPhysics.AddForce(const AOwner: TD3VisualObject; const Force: TD3Vector);
var
  P: TD3Vector;
begin
  if AOwner.Body = 0 then
    Exit;
  if not AOwner.Visible then
    Exit;
  P := AOwner.AbsolutePosition;
  NewtonBodyAddImpulse(NewtonBody(AOwner.Body), @Force, @P);
end;

function TD3NewtonPhysics.ObjectByBody(const Body: integer): TD3VisualObject;
begin
  Result := nil;
  if not FNewtonExists then
    Exit;
  Result := TD3VisualObject(NewtonBodyGetUserData(NewtonBody(Body)));
end;

function TD3NewtonPhysics.CheckWorld(const AOwner: TD3VisualObject): NewtonWorld;
begin
  Result := nil;
  if not FNewtonExists then
    Exit;
  if (AOwner.Projection = d3ProjectionScreen) and (FScreenWorld = 0) then
  begin
    FScreenWorld := CreateWorld;
    Result := NewtonWorld(FScreenWorld);
    Exit;
  end;
  if (AOwner.Projection = d3ProjectionCamera) and (FSpaceWorld = 0) then
  begin
    FSpaceWorld := CreateWorld;
    Result := NewtonWorld(FSpaceWorld);
    Exit;
  end;
  case AOwner.Projection of
    d3ProjectionScreen: Result := Pointer(FScreenWorld);
    d3ProjectionCamera: Result := Pointer(FSpaceWorld);
  end;
end;

function TD3NewtonPhysics.CreateBox(const AOwner: TD3VisualObject; const ASize: TD3Vector): integer;
var
  Collision: NewtonCollision;
  Matrix: TD3Matrix;
  Inertia: TD3Vector;
  World: NewtonWorld;
  Mass: single;
  NJ: NewtonJoint;
begin
  Result := 0;
  if not FNewtonExists then
    Exit;
  World := CheckWorld(AOwner);
  if AOwner.Body <> 0 then
  begin
    Result := 0;
    Exit;
  end;
  NJ := nil;
  Collision := NewtonCreateBox(World, ASize.X, ASize.Y, ASize.Z, 0, nil);
  Matrix := AOwner.AbsoluteMatrix;
  Result := integer(NewtonCreateDynamicBody(World, Collision, @Matrix.M));
  NewtonBodySetUserData(NewtonBody(Result), AOwner);
  if AOwner.Dynamic = True then
    Mass := 1
  else
    Mass := 0;
  Inertia.X := Mass * ASize.Z;
  Inertia.Y := Mass * ASize.Y;
  Inertia.Z := Mass * ScreenDepth;
  NewtonBodySetMassMatrix(NewtonBody(Result), Mass, Inertia.X, Inertia.Y, Inertia.Z);
  NJ := nil;
  if AOwner.Dynamic = True then
  begin
    FNeedSimulation := True;
    NewtonBodySetForceAndTorqueCallback(NewtonBody(Result), @PhysicsApplyForceAndTorque);
    Inertia := d3Vector(0, 0, 1);
    NewtonBodySetAutoSleep(NewtonBody(Result), 1);
    if integer(World) = FScreenWorld then
    begin
      NJ := NewtonConstraintCreateUpVector(World, @Inertia, NewtonBody(Result));
    end;
  end;
  NewtonDestroyCollision(Collision);
  FBodies.Add(Pointer(Result));
  FJoins.Add(NJ);
end;

function TD3NewtonPhysics.CreateCone(const AOwner: TD3VisualObject; const ASize: TD3Vector): integer;
var
  Collision: NewtonCollision;
  Matrix: TD3Matrix;
  Inertia: TD3Vector;
  World: NewtonWorld;
  Mass: single;
  NJ: NewtonJoint;
begin
  Result := 0;
  if not FNewtonExists then
    Exit;
  World := CheckWorld(AOwner);
  if AOwner.Body <> 0 then
  begin
    Result := 0;
    Exit;
  end;
  Collision := NewtonCreateCone(World, ASize.X / 2, ASize.Z, 1, nil);
  Matrix := AOwner.AbsoluteMatrix;
  Result := integer(NewtonCreateDynamicBody(World, Collision, @Matrix.M));
  NewtonBodySetUserData(NewtonBody(Result), AOwner);
  if AOwner.Dynamic = True then
    Mass := 1
  else
    Mass := 0;
  Inertia.X := Mass * ASize.Z;
  Inertia.Y := Mass * ASize.Y;
  Inertia.Z := Mass * ScreenDepth;
  NewtonBodySetMassMatrix(NewtonBody(Result), Mass, Inertia.X, Inertia.Y, Inertia.Z);
  NJ := nil;
  if AOwner.Dynamic = True then
  begin
    FNeedSimulation := True;
    NewtonBodySetForceAndTorqueCallback(NewtonBody(Result), @PhysicsApplyForceAndTorque);
    Inertia := d3Vector(0, 0, 1);
    NewtonBodySetAutoSleep(NewtonBody(Result), 1);
    if integer(World) = FScreenWorld then
    begin
      NJ := NewtonConstraintCreateUpVector(World, @Inertia, NewtonBody(Result));
    end;
  end;
  NewtonDestroyCollision(Collision);
  FBodies.Add(Pointer(Result));
  FJoins.Add(NJ);
end;

function TD3NewtonPhysics.CreateCylinder(const AOwner: TD3VisualObject; const ASize: TD3Vector): integer;
var
  Collision: NewtonCollision;
  Matrix: TD3Matrix;
  Inertia: TD3Vector;
  World: NewtonWorld;
  Mass: single;
  NJ: NewtonJoint;
begin
  Result := 0;
  if not FNewtonExists then
    Exit;
  World := CheckWorld(AOwner);
  if AOwner.Body <> 0 then
  begin
    Result := 0;
    Exit;
  end;
  Collision := NewtonCreateCylinder(World, ASize.X / 2, ASize.Z, 2, nil);
  Matrix := AOwner.AbsoluteMatrix;
  Result := integer(NewtonCreateDynamicBody(World, Collision, @Matrix.M));
  NewtonBodySetUserData(NewtonBody(Result), AOwner);
  if AOwner.Dynamic = True then
    Mass := 1
  else
    Mass := 0;
  Inertia.X := Mass * ASize.Z;
  Inertia.Y := Mass * ASize.Y;
  Inertia.Z := Mass * ScreenDepth;
  NewtonBodySetMassMatrix(NewtonBody(Result), Mass, Inertia.X, Inertia.Y, Inertia.Z);
  NJ := nil;
  if AOwner.Dynamic = True then
  begin
    FNeedSimulation := True;
    NewtonBodySetForceAndTorqueCallback(NewtonBody(Result), @PhysicsApplyForceAndTorque);
    Inertia := d3Vector(0, 0, 1);
    NewtonBodySetAutoSleep(NewtonBody(Result), 1);
    if integer(World) = FScreenWorld then
    begin
      NJ := NewtonConstraintCreateUpVector(World, @Inertia, NewtonBody(Result));
    end;
  end;
  NewtonDestroyCollision(Collision);
  FBodies.Add(Pointer(Result));
  FJoins.Add(NJ);
end;

function TD3NewtonPhysics.CreateSphere(const AOwner: TD3VisualObject; const ASize: TD3Vector): integer;
var
  Collision: NewtonCollision;
  Matrix: TD3Matrix;
  Inertia: TD3Vector;
  World: NewtonWorld;
  Mass: single;
  NJ: NewtonJoint;
begin
  Result := 0;
  if not FNewtonExists then
    Exit;
  World := CheckWorld(AOwner);
  if AOwner.Body <> 0 then
  begin
    Result := 0;
    Exit;
  end;
  Collision := NewtonCreateSphere(World, ASize.X / 2, 3, nil);
  Matrix := AOwner.AbsoluteMatrix;
  Result := integer(NewtonCreateDynamicBody(World, Collision, @Matrix.M));
  NewtonBodySetUserData(NewtonBody(Result), AOwner);
  if AOwner.Dynamic = True then
    Mass := 1
  else
    Mass := 0;
  Inertia.X := Mass * ASize.Z;
  Inertia.Y := Mass * ASize.Y;
  Inertia.Z := Mass * ScreenDepth;
  NewtonBodySetMassMatrix(NewtonBody(Result), Mass, Inertia.X, Inertia.Y, Inertia.Z);
  NJ := nil;
  if AOwner.Dynamic = True then
  begin
    FNeedSimulation := True;
    NewtonBodySetForceAndTorqueCallback(NewtonBody(Result), @PhysicsApplyForceAndTorque);
    Inertia := d3Vector(0, 0, 1);
    NewtonBodySetAutoSleep(NewtonBody(Result), 1);
    if integer(World) = FScreenWorld then
    begin
      NJ := NewtonConstraintCreateUpVector(World, @Inertia, NewtonBody(Result));
    end;
  end;
  NewtonDestroyCollision(Collision);
  FBodies.Add(Pointer(Result));
  FJoins.Add(NJ);
end;

procedure TD3NewtonPhysics.IntDestroyBody(ABody: integer);
var
  Id3: integer;
begin
  if integer(NewtonBodyGetWorld(NewtonBody(ABody))) = FScreenWorld then
  begin
    Id3 := FBodies.IndexOf(Pointer(ABody));
    if Id3 >= 0 then
    begin
      NewtonDestroyBody(NewtonBody(ABody));
      FBodies.Delete(Id3);
      FJoins.Delete(Id3);
    end;
  end
  else
  begin
    NewtonDestroyBody(NewtonBody(ABody));
    FBodies.Remove(NewtonBody(ABody));
  end;
end;

procedure TD3NewtonPhysics.DestroyBody(var ABody: integer);
var
  M: TD3Matrix;
begin
  if ABody = 0 then
    Exit;
  if (FDestroyBodyList <> nil) and (FDestroyBodyList.IndexOf(Pointer(ABody)) >= 0) then
  begin
    ABody := 0;
    Exit;
  end;
  M := IdentityMatrix;
  M.m41 := $FFFFFF;
  M.m42 := $FFFFFF;
  M.m43 := $FFFFFF;
  NewtonBodySetMatrix(NewtonBody(ABody), @M);
  if FDestroyBodyList = nil then
    FDestroyBodyList := TList.Create;
  FDestroyBodyList.Add(Pointer(ABody));
  ABody := 0;
end;

function TD3NewtonPhysics.GetBodyMatrix(const Body: integer): TD3Matrix;
begin
  Result := IdentityMatrix;
  if not FNewtonExists then
    Exit;
  NewtonBodyGetMatrix(NewtonBody(Body), @Result);
end;

procedure TD3NewtonPhysics.SetBodyMatrix(const Body: integer; const M: TD3Matrix);
begin
  if not FNewtonExists then
    Exit;
  NewtonBodySetMatrix(NewtonBody(Body), @M);
end;
